#!/bin/bash

# $Id$
# Copyright (c) 2013 Network Appliance, Inc.
# All rights reserved.


##########################################################################################################
##########################################################################################################
# This script will upgrade vpserver and any other packages shipped with udpate ISO
# Workflow: Client has mounted iso already. This script will stop
# services, wait, upgrade, start vpserver, and wait.
##########################################################################################################
##########################################################################################################
. /lib/lsb/init-functions

PKGNAME=vpupgrader
MAINT_USER=maint
MAINT_GROUP=maintenance
MAINT_HOME="/home/${MAINT_USER}"
MAINT_SCRIPTS="${MAINT_HOME}/scripts"
LOG_DIR="/var/log"
PLATFORM_SETUP_SCRIPTS="${MAINT_HOME}/setup_scripts"
MAINT_APP_RC_DIR="${MAINT_HOME}/app.d"

. ${MAINT_SCRIPTS}/commonfunctions.sh

#
# Variable definitions
#
timestamp=`date +%Y%m%d%H%M%S`
upgradelog="${LOG_DIR}/${PKGNAME}-upgrade-${timestamp}.log"
upgradestatus=0

# Make sure that we trap any control characters so that the user
# does not have access to the shell.
trap logOut 1 2 3 4 5 6 9 11 15


function echo_log()
{
    tstamp=`date +'%b %d %T'`
    echo "$@"
    echo "${tstamp}:  $@" >> ${upgradelog}
}

function log()
{
    tstamp=`date +'%b %d %T'`
    echo "${tstamp}:  $@" &>> ${upgradelog}
}

function ejectUpgrade() {
    # Attempt to clean up any upgrade mounts
	umount -l -f /upgrade &> /dev/null
    umount -l -f /cdrom &> /dev/null
    eject /dev/cdrom &> /dev/null
}

#Message to display when exiting with an error.
function error()
{
    # @todo - change to generic app starter
    #startVP
    echo
    read -n1 -p "Press ENTER to continue." anykey
    echo
    ejectUpgrade
    logOut
}

#
#Add iso in cdrom to sources
#
function sourceISO()
{
    echo "Setting up the ISO..."
    echo

    # First ensure there is nothing mounted on /cdrom
    # Loop is necessary to handle multiple mounts
	umount -l -f /upgrade &> /dev/null
    umount -l -f /cdrom &> /dev/null
    grepResult=`df | grep \/cdrom`
    until [ -z "${grepResult}" ] ; do
        sleep 1
	    umount -l -f /upgrade &> /dev/null
        umount -l -f /cdrom &> /dev/null
        grepResult=`df | grep \/cdrom`
    done

    mount /cdrom &>> ${upgradelog}
    if [[ $? != 0 ]] ; then
        echo_log "ISO mount failed."
        echo
        echo_log "Make sure that a valid ISO is mounted on the virtual CD/DVD device."
	    error
    fi
    
    if [[ -f /cdrom/APPLIANCE_UPGRADE.gpg ]] ; then
        gpg --no-default-keyring --keyring \
            /usr/share/debsig/keyrings/580590ACB277CB74/vplatform-debsig.gpg  \
            --verify /cdrom/APPLIANCE_UPGRADE.gpg /cdrom/APPLIANCE_UPGRADE.bin &>> ${upgradelog}
        
        if [[ $? != 0 ]] ; then
            echo_log "Upgrade Image verification failed."
            echo
            echo_log "Make sure that a valid ISO is mounted on the virtual CD/DVD device."
            error
        fi
    fi
    
    mount -o loop,ro /cdrom/APPLIANCE_UPGRADE.bin /upgrade
    if [[ $? != 0 ]] ; then
        echo_log "Upgrade Image mount failed."
        echo
        echo_log "Make sure that a valid ISO is mounted on the virtual CD/DVD device."
	    error
    fi
    
    # Keep the sources.list tidy
    echo "deb file:/upgrade squeeze main" > /etc/apt/sources.list

    apt-get update &>> ${upgradelog}
    if [[ $? != 0 ]] ; then
        echo_log "Invalid ISO."
        echo
        echo_log "Make sure that a valid ISO is mounted on the virtual CD/DVD device."
	    error
    fi
}

###############################################################################
#
# Ensure installed applications are ready for upgrade
#
###############################################################################
function checkApplicationUpgradeStatus() {
    local rcSum=0
    local msg=""
    
    echo
    echo "Checking installed applications for upgradeability" | tee -a ${upgradelog}
    echo
    for app in $MAINT_APP_RC_DIR/* ; do
    	msg=$($app upgrade-ready)
        rcSum=$(expr ${rcSum} + $?)
        echo "$msg" | tee -a ${upgradelog}
    done
    echo "Accumulated upgradeability check result: $rcSum" >> ${upgradelog}
    
    return $rcSum
}

###############################################################################
#
# Ensure upgrade is run on a vaild greater or equal version iso
#
###############################################################################
function checkUpgradeVersion()
{
    curver=`dpkg -l | grep 'vpupgrader ' | awk '{ print $3 }' | awk -F \- '{ print $1 }'`

    temp=`echo $curver | awk -F . '{ print $1 }'`
    if [[ ! -z $temp ]] ; then
	curmajor=`echo $curver | awk -F . '{ print $1 }' | tr -cd '[0-9]'`
    fi
    temp=`echo $curver | awk -F . '{ print $2 }'`
    if [[ ! -z $temp ]] ; then
	curminor=`echo $curver | awk -F . '{ print $2 }' | tr -cd '[0-9]'`
    fi
    temp=`echo $curver | awk -F . '{ print $3 }'`
    if [[ ! -z $temp ]] ; then
	curmaint=`echo $curver | awk -F . '{ print $3 }' | tr -cd '[0-9]'`
    fi
    log "Current Installed Version: $curver: $curmajor.$curminor.$curmaint"
    echo
    echo "Current installed version: $curver"


    if [ ! -f /upgrade/APPLIANCE_BUILD ]
    then
        echo_log "You have downloaded incorrect ISO."
        echo_log "The ISO you are using for upgrading is not correct."
        echo_log "Make sure that a valid ISO is mounted on the virtual CD/DVD device."
        error
    else
        newver=$(grep build_version /upgrade/APPLIANCE_BUILD | awk -F = '{ print $2 }' | awk -F . '{ print $1"."$2"."$3 }')
        temp=`echo $newver | awk -F . '{ print $1 }'`
        if [[ ! -z $temp ]] ; then
            newmajor=`echo $newver | awk -F . '{ print $1 }' | tr -cd '[0-9]'`
        fi
        temp=`echo $newver | awk -F . '{ print $2 }'`
        if [[ ! -z $temp ]] ; then
            newminor=`echo $newver | awk -F . '{ print $2 }' | tr -cd '[0-9]'`
        fi
        temp=`echo $newver | awk -F . '{ print $3 }'`
        if [[ ! -z $temp ]] ; then
            newmaint=`echo $newver | awk -F . '{ print $3 }' | tr -cd '[0-9]'`
        fi
        log "Upgrade ISO Version: $newver: $newmajor.$newminor.$newmaint."
        echo "Upgrade ISO version: $newver"
        echo

        if [[ 10#${newmajor} -gt 10#${curmajor} ]] || \
           ( [[ 10#${newmajor} -eq 10#${curmajor} ]] && [[ 10#${newminor} -gt 10#${curminor} ]] ) || \
           ( [[ 10#${newmajor} -eq 10#${curmajor} ]] && [[ 10#${newminor} -eq 10#${curminor} ]] && [[ 10#${newmaint} -ge 10#${curmaint} ]] ) ; then
            log "Valid upgrade version"
        else
            echo_log "You have downloaded an incorrect ISO version."
            echo_log "The ISO you are using for upgrade is either from an earlier"
            echo_log "or the same release."
            echo
            echo_log "Make sure that a valid ISO is mounted on the virtual CD/DVD device."
            error
        fi
    fi
}

#
#Stop services and wait
#
#Perform the upgrade/upgrade of services
#
function doUpgrade()
{
    # Move/duplicate our live pre-upgrade.sh, .bashrc and common_functions scripts
    # in case they change out from under us.
    mv ${MAINT_SCRIPTS}/pre-upgrade.sh ${MAINT_SCRIPTS}/pre-upgrade.sh_orig
    cp ${MAINT_SCRIPTS}/pre-upgrade.sh_orig ${MAINT_SCRIPTS}/pre-upgrade.sh
    mv ${MAINT_SCRIPTS}/commonfunctions.sh ${MAINT_SCRIPTS}/commonfunctions.sh_orig
    cp ${MAINT_SCRIPTS}/commonfunctions.sh_orig ${MAINT_SCRIPTS}/commonfunctions.sh
    mv ${MAINT_HOME}/.bash_profile ${MAINT_HOME}/.bash_profile_orig
    cp ${MAINT_HOME}/.bash_profile_orig ${MAINT_HOME}/.bash_profile
    chown ${MAINT_USER}:${MAINT_GROUP} ${MAINT_HOME}/.bash_profile

    echo_log "Verifying virtual appliance upgrade..."
    # Manually verify our key packages
    /usr/bin/debsig-verify /upgrade/dists/squeeze/main/binary-amd64/vpupgrader.deb  &>> ${upgradelog}
    result=$?
    if [[ $result -ne 0 ]] ; then
        echo_log "Verification of vpupgrader package failed."
        error
    fi
    /usr/bin/debsig-verify /upgrade/dists/squeeze/main/binary-amd64/vplatform.deb  &>> ${upgradelog}
    result=$?
    if [[ $result -ne 0 ]] ; then
        echo_log "Verification of vplatform package failed."
        error
    fi
    /usr/bin/debsig-verify /upgrade/dists/squeeze/main/binary-amd64/vmware-tools.deb 2>&1 >> ${upgradelog}
    result=$?
    if [[ $result -ne 0 ]] ; then
        echo_log "Verification of vmware-tools package failed."
        error
    fi
    /usr/bin/debsig-verify /upgrade/dists/squeeze/main/binary-amd64/oracle-jre-installer.deb 2>&1 >> ${upgradelog}
    result=$?
    if [[ $result -ne 0 ]] ; then
        echo_log "Verification of oracle-jre-installer package failed."
        error
    fi
    
    # Update our key platform packages
    echo_log "Upgrading virtual appliance platform scripts..."
    dpkg -iEG /upgrade/dists/squeeze/main/binary-amd64/vpupgrader.deb \
              /upgrade/dists/squeeze/main/binary-amd64/vplatform.deb \
              /upgrade/dists/squeeze/main/binary-amd64/vmware-tools.deb \
              /upgrade/dists/squeeze/main/binary-amd64/oracle-jre-installer.deb \
              2>&1 >> ${upgradelog}
    result=$?
    if [[ $result -ne 0 ]] ; then
        echo
        echo_log "Upgrade virtual appliance platform scripts failed."
        echo_log "Contact customer support and provide the necessary log information."
        echo_log "Log information can be fetched from the following locations:"
        echo_log "* ${upgradelog}"
        error
    fi
    
    # The calling chain from the old .bash_profile->pre-upgrade.sh
    # will remain active.
    # The main upgrade script should be replaced at this point.
    # The new platform script files will be in effect for this
    # file.
    /bin/bash ${MAINT_SCRIPTS}/upgrade-appliance.sh ${upgradelog}
    result=$?
    if [[ $result -eq 0 ]]; then
        echo
        echo_log "Successfully upgraded virtual appliance"
        echo_log "from version $curver to $newver"
        echo
        upgradestatus=1
        ejectUpgrade
    else
        echo
        echo_log "Virtual appliance upgrade failed."
        echo_log "Contact NetApp customer support and provide the necessary log information."
        echo_log "Log information can be fetched from the following locations:"
        echo_log "* ${upgradelog}"
        error
    fi
}
    

# ************ Main section ************

function main() {
    #
    #Warn user that this process will bring down the services
    #
    echo
    echo "Make sure that a valid ISO is mounted on the virtual CD/DVD device."
    echo
	echo "WARNING: The 7.1 and later versions of virtual appliance for VSC, VASA Provider, and SRA require 8GB of RAM to be reserved to work optimally." 
	echo "So, you must reserve 8GB of RAM before starting the upgrade process."
	echo
    #echo "For more information about how to perform the upgrade,"
    #echo "see the Installation and Setup Guide"

    echo
    #echo "Application services will now"
    #echo "be stopped for a couple of minutes."
    #echo "Application services will resume"
    #echo "after completion of the upgrade process."
    echo

    while [[ 1 ]]
    do
        read -r -p "Do you wish to continue? (y/n):" answer
        if [ "$answer" == "n" ]  || [ "$answer" == "N" ]; then
            echo
            echo_log "Virtual appliance upgrade cancelled."
            echo
            logOut
        fi
        if [ "$answer" == "y" ]  || [ "$answer" == "Y" ]; then
            break
        fi
    done
    
    checkApplicationUpgradeStatus
    if [[ $? -ne 0 ]] ; then
        echo
        echo "Application servers not ready for upgrade."
        echo "Please address the application server upgrade prerequisites"
        echo "before starting the upgrade process."
        echo
        read -n1 -p "Press ENTER to continue." anykey
        echo
        exit 100
    fi

    # @todo have to discuss what type of VM backup customer needs to take before starting upgrade.
    echo
    echo "You should make a backup of this virtual appliance"
    echo "before upgrading."
    echo
    while [[ 1 ]]
    do
        read -r -p "Do you wish to continue? (y/n):" answer
        if [ "$answer" == "n" ]  || [ "$answer" == "N" ]; then
            echo
            echo_log "Virtual appliance upgrade cancelled."
            echo
            logOut
        fi
        if [ "$answer" == "y" ]  || [ "$answer" == "Y" ]; then
            break
        fi
    done

    # Wrap the given command so that it puts output to the terminal and to a logfile
    #cmdoutlog="${dest.log.dir}/upgradecmdout"

    checkAvailableSpace 80

    sourceISO

    checkUpgradeVersion

    serverStartStop all stop
    

    doUpgrade ${upgradelog}
    sleep 5

    serverStartStop all start

    # Make sure that on successfull upgrade, logout from the maintenance console.
    # Return code 100 make sure that session logsout from maintenance console.
    # Any return code other than 100 is failure and maintenance console goes to main menu in case of failure.
    #if [[ ${servicestart} -eq 1 && ${upgradestatus} -eq 1 ]]; then
    if [[ ${upgradestatus} -eq 1 ]]; then
        echo_log "Upgrade of virtual appliance is successful."
        echo
        read -n1 -p "Press ENTER to continue." anykey
        echo
        exit 100
    fi
    logOut
}

main "$@"
